<?php
namespace Elementor;

if ( ! defined( 'ABSPATH' ) ) exit;

class TMintroElementor {

    protected static $_instance = null;

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
    }
    
    public function __construct() {
		$this->init_hooks();
	}

    public static function init_hooks() {
        add_action( 'elementor/element/section/section_advanced/after_section_end', [__CLASS__, 'add_section'] );
        add_action( 'elementor/element/column/section_advanced/after_section_end', [__CLASS__, 'add_section'] );
        add_action( 'elementor/element/common/_section_style/after_section_end', [__CLASS__, 'add_section'],1 );
        add_action( 'elementor/element/container/section_layout_container/after_section_end', [__CLASS__, 'add_section'] );
        add_action( 'elementor/frontend/section/before_render', [__CLASS__, 'before_render'] );
        add_action( 'elementor/frontend/column/before_render', [__CLASS__, 'before_render'] );
        add_action( 'elementor/frontend/widget/before_render', [__CLASS__, 'before_render'] );
        add_action( 'elementor/frontend/container/before_render', [__CLASS__, 'before_render'] );
        add_action( 'elementor/element/wp-page/document_settings/after_section_end', [__CLASS__, 'document_settings'] );
        add_action( 'elementor/element/wp-post/document_settings/after_section_end', [__CLASS__, 'document_settings'] );
        add_action( 'wp_footer', [__CLASS__, 'enqueue_scripts'] );
    }

    // Page Settings
    public static function document_settings($page) {
        $page->start_controls_section(
			'_section_tmintro_page',
			[
				'label' => esc_html__( 'TM Intro', 'tmintro' ),
                'tab' => \Elementor\Controls_Manager::TAB_SETTINGS,
			]
		);

        $page->add_control(
            'tmintro_activation',
            [
                'label' => esc_html__( 'Activation', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'enable' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'disable' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'disable',
				'toggle' => false
            ]
        );

        $page->add_control(
			'tmintro_notice',
			[
				'type' => \Elementor\Controls_Manager::RAW_HTML,
                'condition' => ['tmintro_activation' => 'enable'],
				'raw' => esc_html__( 'Please note! The plugin has been disabled on backend to avoid possible issues using Elementor. Please view the page on the frontend to test it.', 'tmintro' ),
				'content_classes' => 'elementor-panel-alert elementor-panel-alert-info',
			]
		);

        $page->add_control(
            'tmintro_pageload',
            [
                'label' => esc_html__( 'Start After Page Load', 'tmintro' ),
                'description'    => esc_html__( 'If you disable this, you will need to create a trigger element (for example; button) and enter "tmintro" to "Advanced -> CSS Classes" field. The intro will start only if you click that button.', 'tmintro'), 
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'enable' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'disable' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'enable',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_cookie',
            [
                'label' => esc_html__( 'Set Cookie', 'tmintro' ),
                'description'    => esc_html__( 'If you enable this, after the user close the instructions, the user onboarding tour will not start again, for that user (until the browser cache is cleared).', 'tmintro'), 
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'true',
                'conditions'   => [
                    'relation' => 'and',
                    'terms' => [
                        [
                            'name'  => 'tmintro_activation',
                            'value' => 'enable',
                        ],
                        [
                            'name'  => 'tmintro_pageload',
                            'value' => 'enable',
                        ]
                    ]
                ],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_exitonesc',
            [
                'label' => esc_html__( 'Exit on ESC', 'tmintro' ),
                'description'    => esc_html__( 'Exit introduction when pressing Escape button.', 'tmintro'), 
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'true',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_exitoverlay',
            [
                'label' => esc_html__( 'Exit on Overlay Click', 'tmintro' ),
                'description'    => esc_html__( 'Exit introduction when clicking on overlay layer.', 'tmintro'), 
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'true',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_fixed_tooltip',
            [
                'label' => esc_html__( 'Fixed Tooltip', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'enable' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'disable' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'disable',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_stepnumbers',
            [
                'label' => esc_html__( 'Show Step Numbers', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'true',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_keyboard',
            [
                'label' => esc_html__( 'Keyboard Navigation', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'false',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_buttons',
            [
                'label' => esc_html__( 'Navigation Buttons', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'true',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_hideprev',
            [
                'label' => esc_html__( 'Hide Previous Button', 'tmintro' ),
                'description'    => esc_html__( 'Hide the previous button in the first step? Otherwise, it will render a disabled button.', 'tmintro'), 
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'false',
                'conditions'   => [
                    'relation' => 'and',
                    'terms' => [
                        [
                            'name'  => 'tmintro_activation',
                            'value' => 'enable',
                        ],
                        [
                            'name'  => 'tmintro_buttons',
                            'value' => 'true',
                        ]
                    ]
                ],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_hidenext',
            [
                'label' => esc_html__( 'Hide Next Button', 'tmintro' ),
                'description'    => esc_html__( 'Hide the next button in the last step? Otherwise, it will render a disabled button. (Note: this will also hide the "Done" button)', 'tmintro'), 
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'false',
                'conditions'   => [
                    'relation' => 'and',
                    'terms' => [
                        [
                            'name'  => 'tmintro_activation',
                            'value' => 'enable',
                        ],
                        [
                            'name'  => 'tmintro_buttons',
                            'value' => 'true',
                        ]
                    ]
                ],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_bullets',
            [
                'label' => esc_html__( 'Navigation Bullets', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'true',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_progress',
            [
                'label' => esc_html__( 'Progress Bar', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'false',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->add_control(
            'tmintro_interaction',
            [
                'label' => esc_html__( 'Disable Interaction', 'tmintro' ),
                'description' => esc_html__( 'Enable/disable interactions with elements inside the highlighted box.', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'true',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->add_control(
			'tmintro_scroll_padding',
			[
				'label' => esc_html__( 'Scroll Top Padding (px)', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 0,
				'max' => 1000,
				'step' => 1,
                'default' => 30,
                'condition' => ['tmintro_activation' => 'enable']
			]
		);

        $page->add_control(
			'tmintro_scroll_speed',
			[
				'label' => esc_html__( 'Scroll Speed (ms)', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 500,
				'max' => 10000,
				'step' => 100,
                'default' => 1000,
                'condition' => ['tmintro_activation' => 'enable']
			]
		);

        $page->add_control(
            'tmintro_scroll_top',
            [
                'label' => esc_html__( 'Scroll To Top', 'tmintro' ),
                'description' => esc_html__( 'Scroll to top of the page after click "done".', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'true' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'false' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'false',
                'condition' => ['tmintro_activation' => 'enable'],
				'toggle' => false
            ]
        );

        $page->end_controls_section();

        $page->start_controls_section(
			'_section_tmintro_page_style',
			[
				'label' => esc_html__( 'TM Intro', 'tmintro' ),
                'tab' => \Elementor\Controls_Manager::TAB_STYLE,
                'condition' => ['tmintro_activation' => 'enable']
			]
		);

        $page->add_control(
            'tmintro_overlay_bg',
            [
                'label' => esc_html__( 'Overlay Background', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-helperLayer' => 'box-shadow: rgba(0, 0, 0, 0) 0px 0px 1px 2px, {{VALUE}} 0px 0px 0px 5000px !important',
                ],
            ]
        );

        $page->add_control(
			'tmintro_highlight_title',
			[
				'label' => esc_html__( 'Highlighted Box', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before'
			]
		);

        $page->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'tmintro_highlighted_border',
				'label' => esc_html__( 'Border', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-helperLayer:before'
			]
		);

        $page->add_control(
			'tmintro_highlighted_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmintro' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .introjs-helperLayer' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .introjs-helperLayer:before' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
                ]
			]
		);

        $page->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'tmintro_highlighted_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-helperLayer:before'
			]
		);

        $page->end_controls_section();

        $page->start_controls_section(
			'_section_tmintro_page_style_2',
			[
				'label' => esc_html__( 'TM Intro (Step Number)', 'tmintro' ),
                'tab' => \Elementor\Controls_Manager::TAB_STYLE,
                'condition' => ['tmintro_activation' => 'enable']
			]
		);

        $page->add_responsive_control(
			'tmintro_step_number_size',
			[
				'label' => esc_html__( 'Size (px)', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 10,
				'max' => 500,
				'step' => 1,
                'default' => 34,
                'selectors' => [
                    '{{WRAPPER}} .introjs-stepNumberLayer' => 'width: {{VALUE}}px;height: {{VALUE}}px;line-height: {{VALUE}}px;',
                ],
			]
		);

        $page->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'tmintro_step_number_typography',
				'selector' => '{{WRAPPER}} .introjs-stepNumberLayer',
			]
		);

        $page->add_control(
            'tmintro_step_number_color',
            [
                'label' => esc_html__( 'Color', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-stepNumberLayer' => 'color: {{VALUE}}',
                ],
            ]
        );

        $page->add_control(
            'tmintro_step_number_bg',
            [
                'label' => esc_html__( 'Background Color', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-stepNumberLayer' => 'background: {{VALUE}}',
                ],
            ]
        );

        $page->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'tmintro_step_number_border',
				'label' => esc_html__( 'Border', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-stepNumberLayer'
			]
		);

        $page->add_control(
			'tmintro_step_number_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmintro' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .introjs-stepNumberLayer' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
                ]
			]
		);

        $page->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'tmintro_step_number_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-stepNumberLayer'
			]
		);

        $page->add_responsive_control(
			'tmintro_step_number_margin',
			[
				'label' => esc_html__( 'Margin', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .introjs-stepNumberLayer' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

        $page->end_controls_section();

        $page->start_controls_section(
			'_section_tmintro_page_style_3',
			[
				'label' => esc_html__( 'TM Intro (Tooltip)', 'tmintro' ),
                'tab' => \Elementor\Controls_Manager::TAB_STYLE,
                'condition' => ['tmintro_activation' => 'enable']
			]
		);

        $page->add_responsive_control(
			'tmintro_tooltip_width',
			[
				'label' => esc_html__( 'Max. Width (px)', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 100,
				'max' => 2000,
				'step' => 10,
                'default' => 300,
                'selectors' => [
                    '{{WRAPPER}} .introjs-tooltip' => 'max-width: {{VALUE}}px;',
                ],
                'condition' => ['tmintro_fixed_tooltip' => 'disable']
			]
		);

        $page->add_control(
            'tmintro_tooltip_bg_color',
            [
                'label' => esc_html__( 'Tooltip Background', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-tooltip' => 'background-color: {{VALUE}}',
                    '{{WRAPPER}} .introjs-arrow.top,{{WRAPPER}} .introjs-arrow.top-right,{{WRAPPER}} .introjs-arrow.top-middle' => 'border-bottom-color: {{VALUE}}',
                    '{{WRAPPER}} .introjs-arrow.bottom,{{WRAPPER}} .introjs-arrow.bottom-right,{{WRAPPER}} .introjs-arrow.bottom-middle' => 'border-top-color: {{VALUE}}',
                    '{{WRAPPER}} .introjs-arrow.right,{{WRAPPER}} .introjs-arrow.right-bottom' => 'border-left-color: {{VALUE}}',
                    '{{WRAPPER}} .introjs-arrow.left,{{WRAPPER}} .introjs-arrow.left-bottom' => 'border-right-color: {{VALUE}}',
                    '{{WRAPPER}} .introjs-custom .introjs-arrow' => 'border-bottom-color: {{VALUE}} !important',
                ],
            ]
        );

        $page->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'tmintro_tooltip_border',
				'label' => esc_html__( 'Border', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-tooltip'
			]
		);

        $page->add_control(
			'tmintro_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmintro' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .introjs-tooltip' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
                ],
                'condition' => ['tmintro_fixed_tooltip' => 'disable']
			]
		);

        $page->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'tmintro_tooltip_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-tooltip',
                'condition' => ['tmintro_fixed_tooltip' => 'disable']
			]
		);

        $page->end_controls_section();

        $page->start_controls_section(
			'_section_tmintro_page_style_4',
			[
				'label' => esc_html__( 'TM Intro (Tooltip Content)', 'tmintro' ),
                'tab' => \Elementor\Controls_Manager::TAB_STYLE,
                'condition' => ['tmintro_activation' => 'enable']
			]
		);

        $page->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'tmintro_title_typography',
                'label' => esc_html__( 'Title Typography', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-tooltip-title',
			]
		);

        $page->add_control(
            'tmintro_title_color',
            [
                'label' => esc_html__( 'Title Color', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-tooltip-title,{{WRAPPER}} .introjs-skipbutton' => 'color: {{VALUE}}',
                ],
            ]
        );

        $page->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'tmintro_text_typography',
                'label' => esc_html__( 'Text Typography', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-tooltiptext',
			]
		);

        $page->add_control(
            'tmintro_text_color',
            [
                'label' => esc_html__( 'Text Color', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-tooltiptext' => 'color: {{VALUE}}',
                ],
            ]
        );

        $page->add_group_control(
			Group_Control_Typography::get_type(),
			[
                'label' => esc_html__( 'Step Info Typography', 'tmintro' ),
				'name' => 'tmintro_step_info_typography',
				'selector' => '{{WRAPPER}} .introjs-helperNumberLayer',
			]
		);

        $page->add_control(
            'tmintro_step_info_color',
            [
                'label' => esc_html__( 'Step Info Color', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-helperNumberLayer' => 'color: {{VALUE}}',
                ],
            ]
        );

        $page->add_control(
            'tmintro_divider_color',
            [
                'label' => esc_html__( 'Divider Color', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-tooltip-header' => 'border-bottom-color: {{VALUE}}',
                    '{{WRAPPER}} .introjs-tooltipbuttons' => 'border-top-color: {{VALUE}}',
                ],
            ]
        );

        $page->end_controls_section();

        $page->start_controls_section(
			'_section_tmintro_page_style_5',
			[
				'label' => esc_html__( 'TM Intro (Navigation)', 'tmintro' ),
                'tab' => \Elementor\Controls_Manager::TAB_STYLE,
                'condition' => ['tmintro_activation' => 'enable']
			]
		);

        $page->add_control(
			'tmintro_buttons_title',
			[
				'label' => esc_html__( 'Buttons', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'none',
			]
		);

        $page->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'tmintro_btn_typography',
                'label' => esc_html__( 'Typography', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-button',
			]
		);

        $page->add_control(
			'tmintro_btn_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 

        $page->start_controls_tabs( 'tabs_btn_style' );
        
        $page->start_controls_tab(
			'tab_btn_normal',
			[
				'label' => esc_html__( 'Normal', 'tmintro'),
			]
		);
        
        $page->add_control(
			'tmintro_btn_text_color',
			[
				'label' => esc_html__( 'Text Color', 'tmintro' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .introjs-button' => 'color: {{VALUE}};',
				]
			]
		);
        
        $page->add_control(
			'tmintro_btn_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmintro' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .introjs-button' => 'background-color: {{VALUE}};',
				]
			]
		);

        $page->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'tmintro_btn_border',
				'label' => esc_html__( 'Border', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-button'
			]
		);
        
        $page->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'tmintro_btn_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-button'
			]
		);
        
        $page->end_controls_tab();

		$page->start_controls_tab(
			'tab_btn_hover',
			[
				'label' => esc_html__( 'Hover', 'tmintro'),
			]
		);

        $page->add_control(
			'tmintro_btn_text_color_hover',
			[
				'label' => esc_html__( 'Text Color', 'tmintro' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .introjs-button:hover' => 'color: {{VALUE}};',
				]
			]
		);
        
        $page->add_control(
			'tmintro_btn_bg_color_hover',
			[
				'label' => esc_html__( 'Background Color', 'tmintro' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .introjs-button:hover' => 'background-color: {{VALUE}};',
				]
			]
		);

        $page->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'tmintro_btn_border_hover',
				'label' => esc_html__( 'Border', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-button:hover'
			]
		);
        
        $page->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'tmintro_btn_border_shadow_hover',
				'label' => esc_html__( 'Box Shadow', 'tmintro' ),
				'selector' => '{{WRAPPER}} .introjs-button:hover'
			]
		);

        $page->end_controls_tab();
        $page->end_controls_tabs();

        $page->add_control(
			'tmintro_btn_hr_2',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 

        $page->add_control(
			'tmintro_btn_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmintro' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .introjs-button' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);

        $page->add_responsive_control(
			'tmintro_btn_padding',
			[
				'label' => esc_html__( 'Padding', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .introjs-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

        $page->add_control(
			'tmintro_bullets_title',
			[
				'label' => esc_html__( 'Bullets', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

        $page->add_control(
            'tmintro_bullets_color',
            [
                'label' => esc_html__( 'Color', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-bullets ul li a' => 'background: {{VALUE}}',
                ],
            ]
        );

        $page->add_control(
            'tmintro_bullets_active',
            [
                'label' => esc_html__( 'Active Color', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-bullets ul li a.active' => 'background: {{VALUE}}',
                ],
            ]
        );

        $page->end_controls_section();

        $page->start_controls_section(
			'_section_tmintro_page_style_6',
			[
				'label' => esc_html__( 'TM Intro (Progress)', 'tmintro' ),
                'tab' => \Elementor\Controls_Manager::TAB_STYLE,
                'condition' => ['tmintro_activation' => 'enable']
			]
		);

        $page->add_responsive_control(
			'tmintro_progress_height',
			[
				'label' => esc_html__( 'Height (px)', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 100,
				'step' => 1,
                'selectors' => [
                    '{{WRAPPER}} .introjs-progress' => 'height: {{VALUE}}px;',
                    '{{WRAPPER}} .introjs-progressbar' => 'line-height: {{VALUE}}px;font-size: {{VALUE}}px;',
                ]
			]
		);

        $page->add_control(
            'tmintro_progress_bg',
            [
                'label' => esc_html__( 'Background Color', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-progress' => 'background-color: {{VALUE}}',
                ],
            ]
        );

        $page->add_control(
            'tmintro_progress_bg_current',
            [
                'label' => esc_html__( 'Current Step Background', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .introjs-progressbar' => 'background-color: {{VALUE}}',
                ],
            ]
        );

        $page->end_controls_section();
    }

    // Enqueue Frontend Scripts & Styles Only If Required
    public static function enqueue_scripts() {
        // Get the Settings
        $post_id = get_the_ID();
        $page_settings_manager = \Elementor\Core\Settings\Manager::get_settings_managers( 'page' );
        $page_settings_model = $page_settings_manager->get_model( $post_id );

        $activation = $page_settings_model->get_settings( 'tmintro_activation' );
        $pageLoad = $page_settings_model->get_settings( 'tmintro_pageload' );
        $cookieEnable = $page_settings_model->get_settings( 'tmintro_cookie' );
        $hidePrev = $page_settings_model->get_settings( 'tmintro_hideprev' );
        $hideNext = $page_settings_model->get_settings( 'tmintro_hidenext' );
        $exitOnEsc = $page_settings_model->get_settings( 'tmintro_exitonesc' );
        $exitOnOverlayClick = $page_settings_model->get_settings( 'tmintro_exitoverlay' );
        $showStepNumbers = $page_settings_model->get_settings( 'tmintro_stepnumbers' );
        $keyboardNavigation = $page_settings_model->get_settings( 'tmintro_keyboard' );
        $showButtons = $page_settings_model->get_settings( 'tmintro_buttons' );
        $showBullets = $page_settings_model->get_settings( 'tmintro_bullets' );
        $showProgress = $page_settings_model->get_settings( 'tmintro_progress' );
        $scrollPadding = $page_settings_model->get_settings( 'tmintro_scroll_padding' );
        $disableInteraction = $page_settings_model->get_settings( 'tmintro_interaction' );
        $fixedTooltip = $page_settings_model->get_settings( 'tmintro_fixed_tooltip' );
        $scrollSpeed = $page_settings_model->get_settings( 'tmintro_scroll_speed' );
        $scrollTop = $page_settings_model->get_settings( 'tmintro_scroll_top' );

        if (!empty($activation) && $activation == 'enable') {
            // Enqueue Scripts
            wp_enqueue_style('elementor-icons-fa-solid');
            wp_enqueue_style('tmintro');
            wp_enqueue_script('tmintro');
            wp_enqueue_script('js-cookie');
            wp_enqueue_script('tmintro-custom');

            $tmintro_param = array(
                "init" => $pageLoad,
                "nextLabel" => esc_html__( 'Next', 'tmintro' ),
                "prevLabel" => esc_html__( 'Prev', 'tmintro' ),
                "skipLabel" => '<i class="fas fa-times"></i>',
                "doneLabel" => esc_html__( 'Done!', 'tmintro' ),
                "hidePrev" => $hidePrev,
                "hideNext" => $hideNext,
                "exitOnEsc" => $exitOnEsc,
                "exitOnOverlayClick" => $exitOnOverlayClick,
                "showStepNumbers" => $showStepNumbers,
                "keyboardNavigation" => $keyboardNavigation,
                "showButtons" => $showButtons,
                "showBullets" => $showBullets,
                "showProgress" => $showProgress,
                "scrollPadding" => $scrollPadding,
                "disableInteraction" => $disableInteraction,
                "fixedTooltip" => $fixedTooltip,
                "scrollSpeed" => $scrollSpeed,
                "scrollTop" => $scrollTop,
                "cookieEnable" => $cookieEnable
            );
            
            wp_localize_script('tmintro-custom', 'tmintro_vars', $tmintro_param); 
        }
    }
    
    // Add Section
    public static function add_section( Element_Base $element ) {

        $element->start_controls_section(
            '_section_tmintro',
            [
                'label' => esc_html__( 'TM Intro', 'tmintro' ),
                'tab' => \Elementor\Controls_Manager::TAB_ADVANCED
            ]
        );

        $element->add_control(
            'tmintro_init',
            [
                'label' => esc_html__( 'TM Intro', 'tmintro' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
					'enable' => [
						'title' => esc_html__( 'Enable', 'tmintro' ),
						'icon' => 'eicons eicon-check',
					],
					'disable' => [
						'title' => esc_html__( 'Disable', 'tmintro' ),
						'icon' => 'eicons eicon-close',
					],
				],
				'default' => 'disable',
				'toggle' => false
            ]
        );

        $element->add_control(
			'tmintro_title',
			[
				'label' => esc_html__( 'Tooltip Title', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => esc_html__( 'Hello!', 'tmintro' ),
                'label_block' => true,
                'condition' => ['tmintro_init' => 'enable'],
                'dynamic' => [
					'active' => true,
				]
			]
		);

        $element->add_control(
			'tmintro_text',
			[
				'label' => esc_html__( 'Tooltip Text', 'tmintro' ),
				'type' => \Elementor\Controls_Manager::TEXTAREA,
				'default' => '',
                'condition' => ['tmintro_init' => 'enable'],
                'dynamic' => [
					'active' => true,
				]
			]
		);

        $element->add_control(
			'tmintro_step',
			[
				'label' => esc_html__( 'Step', 'tmintro' ),
                'description' => esc_html__( 'Define the number (priority) of step (optional).', 'tmintro'),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 100,
				'step' => 1,
                'condition' => ['tmintro_init' => 'enable']
			]
		);

        $element->add_control(
			'tmintro_position',
			[
				'label' => esc_html__( 'Tooltip Position', 'tmintro' ),
				'type' => Controls_Manager::SELECT,
				'options' => [
                    'bottom-left-aligned' => esc_html__( 'Bottom Left Aligned', 'tmintro' ),
                    'bottom-middle-aligned' => esc_html__( 'Bottom Center Aligned', 'tmintro' ),
                    'bottom-right-aligned' => esc_html__( 'Bottom Right Aligned', 'tmintro' ),
                    'right' => esc_html__( 'Right', 'tmintro' ),
                    'left' => esc_html__( 'Left', 'tmintro' ),
                    'top-left-aligned' => esc_html__( 'Top Left Aligned', 'tmintro' ),
                    'top-middle-aligned' => esc_html__( 'Top Center Aligned', 'tmintro' ),
                    'top-right-aligned' => esc_html__( 'Top Right Aligned', 'tmintro' )
				],
				'default' => 'bottom-left-aligned',
                'condition' => ['tmintro_init' => 'enable']
			]
		);

        $element->end_controls_section();
        
    }

    // Before Render
    public static function before_render( Element_Base $element ) {
        $settings = $element->get_settings();

        // Add Render Attribute
        if ($settings['tmintro_init'] == 'enable') {
            $element->add_render_attribute(
                '_wrapper', 'class', [
                    'tmintro-wrapper'
                ]
            );
            $element->add_render_attribute(
                '_wrapper', 'data-intro', [
                    esc_attr($settings['tmintro_text'])
                ]
            );
            if (!empty($settings['tmintro_title'])) {
                $element->add_render_attribute(
                    '_wrapper', 'data-title', [
                        esc_attr($settings['tmintro_title'])
                    ]
                );
            }
            if (!empty($settings['tmintro_position'])) {
                $element->add_render_attribute(
                    '_wrapper', 'data-position', [
                        esc_attr($settings['tmintro_position'])
                    ]
                );
            }
            if (!empty($settings['tmintro_step'])) {
                $element->add_render_attribute(
                    '_wrapper', 'data-step', [
                        esc_attr($settings['tmintro_step'])
                    ]
                );
            }
        }
    }

}

/**
 * Returns the main instance of the class
 */

function TMintroElementor() {  
	return TMintroElementor::instance();
}
// Global for backwards compatibility.
$GLOBALS['TMintroElementor'] = TMintroElementor();