(function( $ ) {
    "use strict";

    // Set Options
    var tmintro = introJs().setOptions({ 
        'nextLabel': tmintro_vars.nextLabel,
        'prevLabel': tmintro_vars.prevLabel,
        'skipLabel': tmintro_vars.skipLabel,
        'doneLabel': tmintro_vars.doneLabel,
        'hidePrev': (tmintro_vars.hidePrev === 'true'),
        'hideNext': (tmintro_vars.hideNext === 'true'),
        'nextToDone': true,
        'exitOnEsc': (tmintro_vars.exitOnEsc === 'true'),
        'exitOnOverlayClick': (tmintro_vars.exitOnOverlayClick === 'true'),
        'showStepNumbers': (tmintro_vars.showStepNumbers === 'true'),
        'keyboardNavigation': (tmintro_vars.keyboardNavigation === 'true'),
        'showButtons': (tmintro_vars.showButtons === 'true'),
        'showBullets': (tmintro_vars.showBullets === 'true'),
        'showProgress': (tmintro_vars.showProgress === 'true'),
        'scrollToElement': false,
        'scrollPadding': 0,
        'autoPosition': false,
        'disableInteraction': (tmintro_vars.disableInteraction === 'true'),
        'helperElementPadding': 0
    });

    // Tooltip Resize
    var tooltipResize = function() {
        if ($('body').find('.introjs-helperLayer').offset()) {
            // Settimeout - Wait for the animation end
            setTimeout(function(){
                var ww = document.body.clientWidth;
                var targetLeft = $('body').find('.introjs-helperLayer').offset().left;
                var targetRight = $('body').find('.introjs-helperLayer').offset().left + $('.introjs-helperLayer').outerWidth();
                var tooltipWidth = $('body').find('.introjs-tooltip').outerWidth(true);
                if (tooltipWidth >= targetLeft) {
                    $('body').find('.introjs-left').removeClass('introjs-left').addClass('introjs-custom').css('visibility', 'visible');
                }
                if (tooltipWidth >= (ww - targetRight)) {
                    $('body').find('.introjs-right').removeClass('introjs-right').addClass('introjs-custom').css('visibility', 'visible');
                }
                $('body').removeClass('tmintro-temp');
            }, parseInt(tmintro_vars.scrollSpeed));
        }
    };

    // Elementor Init
    $( window ).on( 'elementor/frontend/init', function() {
        // Check the cookie setting
        if (tmintro_vars.cookieEnable !== 'true' && Cookies.get('tmintro')) {
            Cookies.remove('tmintro', { path: '' });
        }
        
        // If the tooltip will be fixed, add the required class to the body
        if (tmintro_vars.fixedTooltip == 'enable') {
            $('body').addClass('introjs-fixed');
        }

        // Called after starting a new step of introduction
        tmintro.onafterchange(function(targetElement) {
            if (tmintro_vars.disableInteraction !== 'true') {
                $('body').find(".introjs-helperLayer").css('z-index', 9999998);
            }

            $('html, body').animate({
                scrollTop: $('[data-id="' + targetElement.dataset.id + '"]').offset().top - parseInt(tmintro_vars.scrollPadding)
            }, parseInt(tmintro_vars.scrollSpeed));

            // Fixed tooltip layout
            if (tmintro_vars.fixedTooltip == 'enable' && !$('body').find(".introjs-fixed-left").length && !$('body').find(".introjs-fixed-right").length) {
                $('body').find('.introjs-tooltiptext').wrapAll('<div class="introjs-fixed-left"></div>');  
                $('body').find('.introjs-bullets,.introjs-progress,.introjs-helperNumberLayer,.introjs-arrow,.introjs-tooltipbuttons').wrapAll('<div class="introjs-fixed-right"></div>');
                $('body').find('.introjs-fixed-left,.introjs-fixed-right').wrapAll('<div class="introjs-fixed-wrapper"></div>');  
                $('body').find('.introjs-tooltip').addClass('introjs-tooltip-fixed');
                // Settimeout - Wait for the animation end
                setTimeout(function(){
                    $('body').find('.introjs-tooltip').removeClass('introjs-tooltip-fixed');
                }, parseInt(tmintro_vars.scrollSpeed));
            }

            // Show progress bar
            if (tmintro_vars.showProgress === 'true') {
                $('body.introjs-fixed').find('.introjs-tooltip').css('padding-bottom', '10px');
            }

            // Resize tooltips if fixed tooltip is disabled
            if (tmintro_vars.fixedTooltip != 'enable') {
                tooltipResize();
            }
        });

        // Called before starting a new step of introduction
        tmintro.onbeforechange(function() {
            if (tmintro_vars.fixedTooltip != 'enable') {
                $('body').addClass('tmintro-temp');
            }
        });

        // Called after exit of introduction
        tmintro.onexit(function() {
            if (tmintro_vars.scrollTop === 'true') {
                $("html, body").animate({
                    scrollTop: 0
                }, parseInt(tmintro_vars.scrollSpeed));
            }
            if (tmintro_vars.cookieEnable === 'true') {
                Cookies.set('tmintro', 'disabled', { expires: 365 * 10, sameSite: 'Lax', path: '' });
            }
        });

        // Start on page load
        if (tmintro_vars.init == 'enable') {
            if (tmintro_vars.cookieEnable === 'true') {
                if (!Cookies.get('tmintro')) {
                    setTimeout(function(){
                        tmintro.start();
                    }, 100);
                }
            } else {
                setTimeout(function(){
                    tmintro.start();
                }, 100);
            }
        }
    });

    // Window Resize
    $( window ).on( 'resize orientationchange', function() {
        if(!$('body').hasClass("elementor-editor-active")) {
            if (tmintro_vars.fixedTooltip != 'enable') {
                tooltipResize();
            }
        }
    });

    // Button Trigger
    $('body').find('.tmintro').on('click', function(e) {
        e.preventDefault();
        tmintro.start();
    });
})( jQuery );